SystemOrganization addCategory: #SystemReporter!

Object subclass: #SystemReporter
	instanceVariableNames: 'categories categoriesSelected report tinyBenchmarksResult categoryList testRunner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemReporter'!

!SystemReporter commentStamp: 'laza 1/18/2011 12:04' prior: 0!
SystemReporter offers a window where information about the system is gathered. This can be easily copied to the clipboard and be attached to a bug report for better identification of the context the bug occured in.

To extend the SystemReporter:
	- add a method
		reportXYZ: aStream
	  to the reporting category
	- insert a line
		add: #XYZ method: #reportXYZ
	  to the initialize method
!

!SystemReporter class methodsFor: 'instance creation' stamp: 'GermanArduino 8/26/2011 08:42'!
open
	"^ ToolBuilder open: self new."
	
	"
	openLabel: labelString
	| window pane |
	window := (SystemWindow labelled: labelString) model: self.
	pane := self buildTextMorph.
	window addMorph: pane frame: (0@0 corner: 1@1).
	window openInWorld.
	^ window
	"
	
	"^ UIManager default values: self new."
! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:22'!
add: category method: aSymbol
	^self categoryList add: (self categories add: category -> aSymbol) key! !

!SystemReporter methodsFor: 'building' stamp: 'lr 10/31/2005 15:12'!
buildCategoriesWith: aBuilder
	^ aBuilder pluggableMultiSelectionListSpec new
		model: self;
		list: #categoryList;
		menu: #categoryMenu:;
		getIndex: #categorySelected;
		setIndex: #categorySelected:;
		getSelectionList: #categoryAt:;
		setSelectionList: #categoryAt:put:;
		yourself.! !

!SystemReporter methodsFor: 'building' stamp: 'laza 1/18/2011 11:46'!
buildReportWith: aBuilder
	^ aBuilder pluggableTextSpec new
		model: self;
		menu: #reportMenu:;
		getText: #reportText;
		yourself.! !

!SystemReporter methodsFor: 'building' stamp: 'laza 1/17/2011 12:10'!
buildWith: aBuilder
	| window |
	window := aBuilder pluggableWindowSpec new
		model: self; label: self label; extent: self extent;
		children: (OrderedCollection new 
			add: ((self buildCategoriesWith: aBuilder)
				frame: self categoriesFrame;
				yourself);
			add: ((self buildReportWith: aBuilder)
				frame: self reportFrame;
				yourself);
			yourself);
		yourself.
	^ aBuilder build: window.! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:19'!
categories
	^ categories ifNil: [categories := IdentityDictionary new]! !

!SystemReporter methodsFor: 'building' stamp: 'laza 1/17/2011 11:56'!
categoriesFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 0.25 offset: 0;
		bottomFraction: 1 offset: 0! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/17/2011 12:22'!
categoryAt: anIndex
	^ categoriesSelected includes: (self categoryList at: anIndex ifAbsent: [ ^ false ]).! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/17/2011 12:37'!
categoryAt: anInteger put: aBoolean
	categoriesSelected := categoriesSelected
		perform: (aBoolean ifTrue: [ #copyWith: ] ifFalse: [ #copyWithout: ])
		with: (self categoryList at: anInteger ifAbsent: [ ^ self ]).
	self updateReport! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:16'!
categoryList
	^ categoryList ifNil: [categoryList := OrderedCollection new]! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:57'!
categoryMenu: aMenu
	^ aMenu
		title: 'Categories';
		add: 'Select all' action: #selectAllCategories;
		add: 'Select none' action: #selectNoCategories;
		addLine;
		add: 'Refresh' action: #refresh;
		yourself.! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'lr 10/31/2005 15:11'!
categorySelected
	^ 0! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'lr 10/31/2005 15:12'!
categorySelected: anInteger
	self changed: #categorySelected.! !

!SystemReporter methodsFor: 'accessing-report' stamp: 'laza 1/18/2011 11:50'!
copyReportToClipboard
	Clipboard clipboardText: self reportText.
	UIManager default inform: 'Copied Report to Clipboard'! !

!SystemReporter methodsFor: 'private' stamp: 'laza 1/28/2011 09:57'!
enumerate: aBlock on: aStream
	self enumerate: aBlock startAt: 0 on: aStream! !

!SystemReporter methodsFor: 'private' stamp: 'laza 1/28/2011 10:25'!
enumerate: aBlock startAt: first on: aStream
	"Utilitymethod to enumerate Options or Parameters from first to 1000"
	| idx value |
	idx := first.
	[value := aBlock value: idx.
	value = nil or: [idx > 1000]] whileFalse: [
		aStream
			nextPut: $#;
			nextPutAll: idx printString;
			tab;
			nextPutAll: value; cr.
		idx := idx + 1
	].
	idx = first ifTrue: [aStream nextPutAll: 'none'; cr]! !

!SystemReporter methodsFor: 'accessing-ui' stamp: 'lr 10/8/2005 18:39'!
extent
	^ 640 @ 480! !

!SystemReporter methodsFor: 'printing-report' stamp: 'laza 1/18/2011 08:58'!
header: aString on: aStream
	aStream withAttribute: TextEmphasis bold do: [	
		aStream nextPutAll: aString; cr.
		aString size timesRepeat: [aStream nextPut: $-].
		aStream cr]! !

!SystemReporter methodsFor: 'initialize-release' stamp: 'laza 2/22/2011 13:50'!
initialize
	self
		add: #Image method: #reportImage;
		add: #'Image Parameters' method: #reportImageParameters;
		add: #'Image Sources' method: #reportSources;
		add: #'Image Preferences' method: #reportPreferences;
		add: #'MC Repositories' method: #reportRepositories;
		add: #'MC Working Copies' method: #reportWorkingCopies;
		add: #'VM General' method: #reportVM;
		add: #'VM Options' method: #reportVMOptions;
		add: #'VM Modules' method: #reportModules;
		add: #'VM Parameters' method: #reportVMParameters;
		add: #'VM Stats' method: #reportVMStats.
	Smalltalk os platformName = 'Win32' ifTrue: [
		self
			add: #'VM Configuration' method: #reportWin32VMConfig.
		].
	self
		add: #'OS General' method: #reportOS.
	Smalltalk os platformName = 'Win32' ifTrue: [
		self
			add: #'OS Details' method: #reportWin32OSDetails;
			add: #'Hardware Details' method: #reportWin32HardwareDetails;
			add: #'GFX Hardware Details' method: #reportWin32GFXDetails.
		].
	Smalltalk os osVersion = 'linux' ifTrue: [
		self
			add: #'OS Details' method: #reportLinuxOSDetails
	].
	self add: #'OS Environment' method: #reportOSEnvironment.
	self
		add: #'Tiny Benchmarks' method: #reportTinyBenchmarks;
		add: #'SUnit' method: #reportTestRunner;
		add: #'Debug Log' method: #reportDebugLog.
	categoriesSelected := Set with: #Image with: #'VM General'.
	self updateReport
! !

!SystemReporter methodsFor: 'accessing-ui' stamp: 'laza 1/17/2011 11:47'!
label
	^ 'System Reporter' ! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/31/2011 20:59'!
refresh
	tinyBenchmarksResult := testRunner := nil.
	self updateReport! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/1/2011 09:28'!
reportDebugLog: aStream
	| logFilename logStream contents |
	self header: 'Debug Logfile' on: aStream.
	logFilename := Smalltalk image squeakErrorFileName.
	logStream := nil.
	[
		[
			logStream := FileStream readOnlyFileNamed: logFilename.
			aStream nextPutAll: logStream name; cr; cr.
			contents := logStream contents.
			aStream
				nextPutAll: 
					((contents isNil or: [contents size = 0])
						ifTrue: ['<empty>']
						ifFalse: [contents]);
				cr
		] on: Error do: [:ex |
			aStream
				nextPutAll: (
					ex class = FileDoesNotExistException
						ifTrue: [logFilename, ' not found']
						ifFalse: [ex description]);
				cr
		]
	] ensure: [
		logStream ifNotNil: [logStream close]
	]! !

!SystemReporter methodsFor: 'building' stamp: 'laza 1/17/2011 12:14'!
reportFrame
	^LayoutFrame new
		leftFraction: 0.25 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: 0! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/30/2011 05:04'!
reportImage: aStream
	self header: 'Image' on: aStream.
	aStream 
		nextPutAll: Smalltalk image imageName; cr;
		nextPutAll: SystemVersion current version; cr;
		nextPutAll: Smalltalk image lastUpdateString; cr;
		nextPutAll: Smalltalk image currentChangeSetString; cr
! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 09:51'!
reportImageParameters: aStream
	self header: 'Image Commandline Parameters' on: aStream.
	self enumerate: [:idx | Smalltalk image argumentAt: idx] on: aStream.! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/21/2011 23:57'!
reportLinuxOSDetails: aStream
	self header: 'Operating System Details' on: aStream.
	#(
		'/etc/issue'
		'/etc/lsb-release'
		'/proc/version'
	) do: [:path|
		self writeContentsSafelyFromFile: path on: aStream]! !

!SystemReporter methodsFor: 'accessing-report' stamp: 'laza 1/18/2011 11:49'!
reportMenu: aMenu
	^ aMenu
		title: 'Report';
		add: 'Copy to Clipboard' action: #copyReportToClipboard;
		yourself.! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/18/2011 11:54'!
reportModules: aStream
	self header: 'Loaded VM Modules' on: aStream.
	SmalltalkImage current listLoadedModules asSortedCollection do: [:each | aStream nextPutAll: each; cr].

! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 10:07'!
reportOS: aStream
	self header: 'Operating System/Hardware' on: aStream.
	aStream
		nextPutAll: Smalltalk os platformName; space;
		nextPutAll: Smalltalk os osVersion; space;
		nextPutAll: Smalltalk os platformSubtype; cr
! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/22/2011 13:57'!
reportOSEnvironment: aStream
	| env |
	self header: 'Operating System Environment' on: aStream.
	env := [Smalltalk os environ] on: Error do: [^self].
	env keys asSortedCollection do: [:name |
		aStream nextPutAll: name; nextPut: $=; nextPutAll: (env at: name); cr]
	! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 13:51'!
reportPreferences: aStream
	| booleanPrefs prefs valuePrefs attribute falseTA trueTA |
	prefs := Preferences dictionaryOfPreferences.
	booleanPrefs := prefs select: [:each | each type = #Boolean].
	valuePrefs := prefs select: [:each | each type = #Number or: [each type = #String]].
	trueTA := TextColor black.
	falseTA := TextColor gray.
	booleanPrefs := booleanPrefs asSortedCollection: [:a :b | a name <= b name].  
	self header: 'Boolean Preferences' on: aStream.
	booleanPrefs do: [:each |
		attribute := each preferenceValue ifTrue: [trueTA] ifFalse: [falseTA].
		aStream withAttribute: attribute do: [	
		aStream
			nextPutAll: each name;
			tab;  
			nextPutAll: each preferenceValue printString;
			cr]].
	aStream cr.
	self header: 'Value Preferences' on: aStream.
	valuePrefs do: [:each |
		aStream
			nextPutAll: each name;
			tab;  
			nextPutAll: each preferenceValue printString;
			cr].
! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 12:14'!
reportRepositories: aStream
	self header: 'Monticello Repositories' on: aStream.
	MCRepositoryGroup default repositories do: [:each | aStream nextPutAll: each description; cr]! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/18/2011 11:55'!
reportSources: aStream
	self header: 'Image Sources' on: aStream.
	aStream nextPutAll: SourceFiles class printString; cr.
	SourceFiles do: [:each |
		each ifNotNil: [aStream nextPutAll: each printString; cr]]! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/31/2011 21:16'!
reportTestRunner: aStream
	testRunner ifNil: [| runAllTests |
		runAllTests := UIManager confirm: 'Running all Tests\will take long time' withCRs.
		runAllTests
			ifTrue: [testRunner := TestRunner new runAll]
			ifFalse: [
				categoriesSelected remove: #SUnit. 
				^self changed: #categorySelected]].
	self header: 'SUnit Results' on: aStream.
	aStream nextPutAll: testRunner statusText; cr; cr.
	self header: 'Failed Tests' on: aStream.
	testRunner failedList do: [:each | aStream nextPutAll: each; cr].
	aStream cr.
	self header: 'Errors' on: aStream.
	testRunner errorList do: [:each | aStream nextPutAll: each; cr]
! !

!SystemReporter methodsFor: 'accessing-report' stamp: 'laza 1/18/2011 09:42'!
reportText
	^ (report isNil or: [categoriesSelected isEmpty])
		ifTrue: ['-- Choose any category on the left --']
		ifFalse: [report]! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/31/2011 21:13'!
reportTinyBenchmarks: aStream
	tinyBenchmarksResult ifNil: [
		UIManager inform: 'Running the Benchmarks\will take a few seconds' withCRs.
		tinyBenchmarksResult := 0 tinyBenchmarks].
	self header: 'Tiny Benchmarks' on: aStream.
	aStream nextPutAll: tinyBenchmarksResult; cr! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/15/2011 09:55'!
reportVM: aStream
	self header: 'Virtual Machine' on: aStream.
	aStream
		nextPutAll: (Smalltalk vm vmFileName); cr;
		nextPutAll: (Smalltalk vm vmVersion); cr.
	Smalltalk vm buildDate
		ifNotNilDo: [:string | aStream nextPutAll: string; cr].
	[Smalltalk vm platformSourceVersion
		ifNotNilDo: [:v | aStream nextPutAll: 'platform sources revision ', v; cr]]
			on: Warning do: ["unsupported primitive"].
	[Smalltalk vm interpreterSourceVersion
		ifNotNilDo: [:v | aStream nextPutAll: 'VMMaker versionString ', v; cr]]
			on: Warning do: ["unsupported primitive"].
	[Smalltalk vm interpreterClass
		ifNotNilDo: [:string | aStream nextPutAll: string; cr].
	Smalltalk vm cogitClass
		ifNotNilDo: [:string | aStream nextPutAll: string; cr]
	] on: Error do: ["unsupported primitives"]
! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 09:58'!
reportVMOptions: aStream
	self header: 'Virtual Machine Commandline Options' on: aStream.
	self enumerate: [:idx | Smalltalk vm optionAt: idx] startAt: 1 on: aStream! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 12:43'!
reportVMParameters: aStream
	| vmParameters |
	self header: 'Virtual Machine Parameters' on: aStream.
	vmParameters := Smalltalk vm getVMParameters.
	#(
		1	'end of old-space (0-based, read-only)'
		2	'end of young-space (read-only)'
		3	'end of memory (read-only)'
		4	'allocationCount (read-only)'
		5	'allocations between GCs (read-write)'
		6	'survivor count tenuring threshold (read-write)'
		7	'full GCs since startup (read-only)'
		8	'total milliseconds in full GCs since startup (read-only)'
		9	'incremental GCs since startup (read-only)'
		10	'total milliseconds in incremental GCs since startup (read-only)'
		11	'tenures of surving objects since startup (read-only)'
		12	'specific to the translating VM'
		13  'specific to the translating VM'
		14  'specific to the translating VM'
		15  'specific to the translating VM'
		16  'specific to the translating VM'
		17  'specific to the translating VM'
		18  'specific to the translating VM'
		19  'specific to the translating VM'
		20  'specific to the translating VM'
		21	'root table size (read-only)'
		22	'root table overflows since startup (read-only)'
		23	'bytes of extra memory to reserve for VM buffers, plugins, etc.'
		24	'memory threshold above which shrinking object memory (rw)'
		25	'memory headroom when growing object memory (rw)'
		26  'interruptChecksEveryNms - force an ioProcessEvents every N milliseconds, in case the image  is not calling getNextEvent often (rw)'
		27	'number of times mark loop iterated for current IGC/FGC (read-only) includes ALL marking'
		28	'number of times sweep loop iterated  for current IGC/FGC (read-only)'
		29	'number of times make forward loop iterated for current IGC/FGC (read-only)'
		30	'number of times compact move loop iterated for current IGC/FGC (read-only)'
		31	'number of grow memory requests (read-only)'
		32	'number of shrink memory requests (read-only)'
		33	'number of root table entries used for current IGC/FGC (read-only)'
		34	'number of allocations done before current IGC/FGC (read-only)'
		35	'number of survivor objects after current IGC/FGC (read-only)'
		36  'millisecond clock when current IGC/FGC completed (read-only)'
		37  'number of marked objects for Roots of the world, not including Root Table entries for current IGC/FGC (read-only)'
		38  'milliseconds taken by current IGC  (read-only)'
		39  'Number of finalization signals for Weak Objects pending when current IGC/FGC completed (read-only)'
		40  'VM word size - 4 or 8 (read-only)'
	) pairsDo: [:idx :desc |
		aStream
			nextPut: $#;
			nextPutAll: idx printString;
			tab;
			nextPutAll: (vmParameters at: idx) printString;
			tab;
			nextPutAll: desc;
			cr]! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 12:30'!
reportVMStats: aStream
	self header: 'Virtual Machine Statistics' on: aStream.
	aStream
		nextPutAll: Smalltalk vm vmStatisticsReportString! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/15/2011 09:40'!
reportWin32GFXDetails: aStream
	self header: 'Graphics Hardware Details' on: aStream.
	aStream nextPutAll: (SmalltalkImage current getSystemAttribute: 10003)! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/15/2011 09:40'!
reportWin32HardwareDetails: aStream
	self header: 'Hardware Details' on: aStream.
	aStream nextPutAll: (SmalltalkImage current getSystemAttribute: 10001)! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/15/2011 09:39'!
reportWin32OSDetails: aStream
	self header: 'Operating System Details' on: aStream.
	aStream nextPutAll: (SmalltalkImage current getSystemAttribute: 10002)! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 2/15/2011 10:21'!
reportWin32VMConfig: aStream
	| exePath iniPath |
	self header: 'VM Configuration' on: aStream.
	exePath := Smalltalk vm vmFileName.
	iniPath := (exePath copyUpToLast: $.), '.ini'.
	aStream nextPutAll: iniPath; cr.
	self writeContentsSafelyFromFile: iniPath on: aStream.
! !

!SystemReporter methodsFor: 'reporting' stamp: 'laza 1/28/2011 12:08'!
reportWorkingCopies: aStream
	| list |
	self header: 'Monticello Working Copies' on: aStream.
	list := MCWorkingCopy allManagers asSortedCollection: [:a :b | a name <= b name]  .
	list do: [:each | aStream nextPutAll: each description; cr]! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:40'!
selectAllCategories
	categoriesSelected addAll: categoryList.
	self changed: #categorySelected.
	self updateReport! !

!SystemReporter methodsFor: 'accessing-categories' stamp: 'laza 1/18/2011 11:40'!
selectNoCategories
	categoriesSelected removeAll.
	self changed: #categorySelected.
	self updateReport! !

!SystemReporter methodsFor: 'updating' stamp: 'laza 1/18/2011 11:26'!
updateReport
	report := Text streamContents: [:stream | 
		stream 
			withAttribute: (TextFontReference toFont: ((TextStyle named: 'BitstreamVeraSansMono') fontOfSize: 16))
			do: [
				self categoryList do: [:each |
					(categoriesSelected includes: each) ifTrue: [
						self perform: ((categories at: each), ':') asSymbol with: stream.
						stream cr]]]].
	self changed: #reportText! !

!SystemReporter methodsFor: 'private' stamp: 'laza 2/21/2011 23:57'!
writeContentsSafelyFromFile: osPath on: aStream
	aStream nextPutAll:
		([
			(FileStream readOnlyFileNamed: osPath) upToEnd
		 ] on: Error do: [:ex| ex return: ex printString])! !
